import qualified Data.List as Reference
import Lib
import Test.Tasty
import Test.Tasty.HUnit
import Test.Tasty.QuickCheck

unitTests =
  testGroup
    "Unit tests"
    [ testCase "sort []" $ sort ([] :: [Int]) @?= [],
      testCase "sort [1]" $ sort [1] @?= [1],
      testCase "sort [1,2,3]" $ sort [1, 2, 3] @?= [1, 2, 3],
      testCase "sort [3,2,1]" $ sort [3, 2, 1] @?= [1, 2, 3],
      testCase "sort [1,2,1]" $ sort [1, 2, 1] @?= [1, 1, 2]
    ]

-- properties

propIdempotent :: [Int] -> Bool
propIdempotent list = sort list == sort (sort list)

propSameSize :: [Int] -> Bool
propSameSize list = length list == length (sort list)

propInOrder :: [Int] -> Bool
propInOrder list = and $ zipWith (<=) sortedList (tail sortedList)
  where
    sortedList = sort list

propAsReference list = sort list == Reference.sort list
  where
    types = list :: [Int]

propertyTests =
  testGroup
    "Property test"
    [ testProperty "Idempotent" propIdempotent,
      testProperty "Same length" propSameSize,
      testProperty "In order" propInOrder,
      testProperty "As reference" propAsReference
    ]

allTests = testGroup "All test" [unitTests, propertyTests]

main :: IO ()
main = defaultMain allTests
